
From: Paul Jackson <pj@sgi.com>

The cpuset mems_allowed update code in alloc_pages_current could (in
theory) put a task to sleep that didn't allow sleeping (did not have
__GFP_WAIT flag set).  In the rare circumstance that the current tasks
mems_generation is outofdate compared to the tasks cpuset mems_generation,
this mems_allowed update code needs to grap cpuset_sem, which can sleep.

We avoid this by not trying to update mems_allowed here if we can't sleep
(__GFP_WAIT not set).

Thanks to Ray Bryant <raybry@sgi.com> for noticing this.

Signed-off-by: Paul Jackson <pj@sgi.com>
Signed-off-by: Andrew Morton <akpm@osdl.org>
---

 25-akpm/mm/mempolicy.c |    6 +++++-
 1 files changed, 5 insertions(+), 1 deletion(-)

diff -puN mm/mempolicy.c~cpusets-alloc-gfp_wait-sleep-fix mm/mempolicy.c
--- 25/mm/mempolicy.c~cpusets-alloc-gfp_wait-sleep-fix	2005-03-16 17:12:39.000000000 -0800
+++ 25-akpm/mm/mempolicy.c	2005-03-16 17:12:39.000000000 -0800
@@ -788,12 +788,16 @@ alloc_page_vma(unsigned gfp, struct vm_a
  *	Allocate a page from the kernel page pool.  When not in
  *	interrupt context and apply the current process NUMA policy.
  *	Returns NULL when no page can be allocated.
+ *
+ *	Don't call cpuset_update_current_mems_allowed() unless
+ *	1) it's ok to take cpuset_sem (can WAIT), and
+ *	2) allocating for current task (not interrupt).
  */
 struct page *alloc_pages_current(unsigned gfp, unsigned order)
 {
 	struct mempolicy *pol = current->mempolicy;
 
-	if (!in_interrupt())
+	if ((gfp & __GFP_WAIT) && !in_interrupt())
 		cpuset_update_current_mems_allowed();
 	if (!pol || in_interrupt())
 		pol = &default_policy;
_
